package cz.zcu.fav.kiv.jsimcasestudies.watersystem;

/**
 *  An interface shared between the control program and the model of its environment.
 *  Once defined (together with other abstract interfaces), the control program and the environment model
 *  can be developed in parallel without having any knowledge about each other. The interface defines
 *  methods that the real control program will use in order to communicate with its environment.
 *  The model of environment must provide its own implementation of these methods that (may)
 *  affect or reflect the model's data.
 *
 *  @author Cesar ALLAIN
 *  @author Jarda KACER
 *  @version 1.0 from 2004-03-18
 */

public interface CommonControlInterface
{
	/**
	 *  Reads the Source Level Low flag of a water source.
	 *
	 *  @param sourceNo Number of the source whose flag has to be read.
	 *
	 *  @return The value of the Source Level Low flag of the specified source.
	 */
	public boolean readSourceLevelLow(int sourceNo);

	/**
	 *  Reads the Source Level High flag of a water source.
	 *
	 *  @param sourceNo Number of the source whose flag has to be read.
	 *
	 *  @return The value of the Source Level High flag of the specified source.
	 */
	public boolean readSourceLevelHigh(int sourceNo);

	/**
	 *  Reads the Pump IS On flag of a water source.
	 *
	 *  @param sourceNo Number of the source whose flag has to be read.
	 *
	 *  @return The value of the Pump Is On flag of the specified source.
	 */
	public boolean readPump(int sourceNo);

	/**
	 *  Sets the Pump Is On flag of a water source.
	 *
	 *  @param sourceNo Number of the source whose flag has to be set.
	 *
	 *  @return The old value of the Pump Is On flag of the specified source.
	 */
	public boolean setPump(int sourceNo, boolean newState);

	/**
	 *  Reads the Source Level Low flag of the main water tank.
	 *
	 *  @return The value of the Source Level Low flag of the main water tank.
	 */
	public boolean readWaterStationLevelLow();

	/**
	 *  Reads the Source Level High flag of the main water tank.
	 *
	 *  @return The value of the Source Level High flag of the main water tank.
	 *
	 */
	public boolean readWaterStationLevelHigh();

	/**
	 *  Reads the Output Flow register of the main water tank.
	 *
	 *  @return The value of the Output Flow register of the main water tank.
	 */
	public int readOutputFlow();       

	/**
	 *  Returns the reference to the interrupt controller object.
	 *
	 *  @return The reference to the interrupt controller object.
	 */
	public InterruptControllerInterface getInterruptControllerAddress();       



} // interface CommonControlInterface

