package cz.zcu.fav.kiv.jsimcasestudies.watersystem;

/**
 *  An interface shared between the control program and the model
 *  of its environment.
 *  Methods of the interface are aimed to be called from the control
 *  program side.
 *  The interface must be implemented at the environment model side. Then it serves as 
 *  a model of HW interrupt controller,
 *  i.e. a device that passes interrupts (events to be served)
 *  from the environment  to the control program.
 *
 *  @author St.R.
 *  @version 1.0 from 2004-04-30
 */


public interface InterruptControllerInterface
{
	/**
	 *  Binds the address of an interrupt service routine to a level of interrupt. 
         *  It has to be called from the control program side to define the 
         *  function that will serve the given level of interrupts.
	 *
         *  @param level  Level of interrupts.
         *  @param ie     Interrupt entry (it contains the service routine) 
	 */
	public void bindInterrupt(int level, InterruptEntryInterface ie);


        /**
	 *  It sets mask of the given level of interrupts. Value
         *  "true" of the mask means that interrupts are not served on the
         *  contrary to the "false" value.
	 *
         *  @param level  Level of interrupts.
         *  @param mask   Value of the mask to be set
	 */
	public void maskInterrupt(int level,  boolean mask);
	
} // interface InterruptControllerInterface

