package cz.zcu.fav.kiv.jsimcasestudies.watersystem.application;

import cz.zcu.fav.kiv.jsimcasestudies.watersystem.*;
import cz.zcu.fav.kiv.jsim.*;


/**
 *  An implementation of interrupt entry object.
 *  The implementation assumes that more threads can wait for the 
 *  same interrupt event. 
 * 
 *  !!! This implementation uses JSim directly and it should be changed
 *      to use the JiJ package. 
 * 
 *  @author St. RACEK
 *  @version 1.0 from 2004-04-30
 */

class InterruptEntry implements InterruptEntryInterface{

	private JSimHead waitingRoom;
        private JSimSimulation simulation;
	
	/**
	* Contructor of the class InterruptEntry
	*/

	protected InterruptEntry(JSimSimulation simulation) throws
           JSimInvalidParametersException, JSimTooManyHeadsException
        {
          waitingRoom = new JSimHead ("InterruptQueue", simulation);
          this.simulation = simulation;
        } // constructor


	/**
	* Interrupt service routine - it awakes all waiting
        * processes and empties the waiting room.
	*/

        public void interruptService() 
        {         
          
          JSimLink waitingThread;

          try {
            while (true) {                           
              if (waitingRoom.empty()) return;                       
              waitingThread = waitingRoom.first();
              if (waitingThread != null)                
              ((JSimProcess)waitingThread.getData()).activate(simulation.getCurrentTime());                          
              waitingThread.out();
            } // while           
          } //
          catch (JSimException e) {
            e.printStackTrace();
            e.printComment(System.err);
          }
        } 	
    

	/**
	* Control processes calls the method when they wants to wait
        *   for the (one common) interrupt. The calling process is put into the list.
        *   and it must be passivated after it!
        * 
        *   @param process Process that will to wait
        *
	*/
       
        public void waitInterrupt(JSimProcess process) throws JSimSecurityException {
          JSimLink linkProcess = new JSimLink(process);
          linkProcess.into(waitingRoom);    
        }  


} // InterruptEntry
