package cz.zcu.fav.kiv.jsimcasestudies.watersystem.application;

import cz.zcu.fav.kiv.jsimcasestudies.watersystem.*;

/**
 *  An encapsulation of a water source data and functionality.
 *
 *  This source code needn`t to be changed when passing it to the
 *     production version.
 *
 *  @author Cesar ALLAIN
 *  @author Jarda KACER
 *  @author St.RACEK
 *  @version 1.0 from 2004-03-20
 */


class SourceMonitor
{
	/**
	*  An interface between the controlled object and
        *  the control program
	*/
	private CommonControlInterface controlInterface;

	/**
	* Number of the source
	*/
        private final int sourceNumber;

        /**
        * Constructor of the class
        * @param controlInterface
        * @param sourceNumber
        */

        protected SourceMonitor (CommonControlInterface controlInterface,
          int sourceNumber)
	{
	  this.controlInterface = controlInterface;
          this.sourceNumber = sourceNumber;
	} // constructor

	/**
	* Gets the number of the source
	*/
	protected int getNumber()
	{
		return sourceNumber;
	} // getNumber

	/**
	* Synchronized method that switches on the pump
	*/
	synchronized protected void pumpSwitchOn()
	{
          if (!controlInterface.readPump(sourceNumber))
            controlInterface.setPump(sourceNumber, true);
	} // pumpSwitchOn


	/**
	* Synchronized method that shows if the pump is switched off
	*/
        synchronized protected void pumpSwitchOff()
	{
          if (controlInterface.readPump(sourceNumber))
            controlInterface.setPump(sourceNumber, false);
	} // pumpSwitchOn

	/**
	* Synchronized method that returns consistent state of the source
	*/
        synchronized protected SourceState getSourceState()
	{
          SourceState state = new SourceState();
          state.levelLow = controlInterface.readSourceLevelLow(sourceNumber);
          state.levelHigh = controlInterface.readSourceLevelHigh(sourceNumber);
          state.pumpIsOn = controlInterface.readPump(sourceNumber);
          return state;

	} // getSourceState

} // class SourceMonitor

