package cz.zcu.fav.kiv.jsimcasestudies.watersystem.application;

import cz.zcu.fav.kiv.jsimcasestudies.watersystem.*;


/**
 *  An encapsulation of the water station data and functionality.
 *
 *  !!! This code should be changed in order to use the JiJ package
 *  - passive waiting in requestActivity().
 *
 *  @author Cesar ALLAIN
 *  @author Jarda KACER
 *  @author St.RACEK
 *  @version 1.0 from 2004-03-21
 */


class StationMonitor
{
	/**
	*  A common interface between controlled object and the
        *  control program
	*
	*/
	private CommonControlInterface controlInterface;

	/**
	* Maximum number of running pumps
	*/

        private final int maxNumberOfPumpsOn;

	/**
	*  Count of pumps currently switched on
	*/

	public int cntPumpsOn;

	/**
	*  Count of pumps that are ready to work
	*/

        public int cntPumpsReady; // not updated here

	/**
	*  Constructor StationMonitor
	*  @param controlInterface
	*  @param maxNumberOfPumpsOn
	*/

        protected StationMonitor (CommonControlInterface controlInterface,
          int maxNumberOfPumpsOn)
	{
	  this.controlInterface = controlInterface;
          this.maxNumberOfPumpsOn = maxNumberOfPumpsOn;
          cntPumpsOn = 0;
          cntPumpsReady = 0;

	} // constructor

	/**
	*  Synchronized method - source reports its ability
        *     to start the pump and requests the activity "permit"
        *     to be granted
	*/

	synchronized protected boolean requestActivity()
	{
          if ((cntPumpsOn < maxNumberOfPumpsOn) && (!controlInterface.readWaterStationLevelHigh())) {
            cntPumpsOn++;
            return true;
          }
          else return false;
	} // requestActivity

	/**
	*  Synchronized method - source reports, that its pump
        *    has been switchd off (water below low level)
        */

        synchronized protected void releaseActivity()
	{
          if (cntPumpsOn > 0)
            cntPumpsOn--;
          else
            cntPumpsOn = 0;
	} // releaseActivity

	/**
	*  Synchronized method that return count of running pumps
	*/

        synchronized protected int getCntPumpsOn () {
          return cntPumpsOn;
        }

	/**
	*  Synchronized method that return consistent state of the station
	*/

        synchronized protected StationState getStationState()
	{
          StationState state = new StationState();
          state.levelLow = controlInterface.readWaterStationLevelLow();
          state.levelHigh = controlInterface.readWaterStationLevelHigh();
          state.cntPumpsOn = cntPumpsOn;
          state.cntPumpsReady = cntPumpsReady;
          state.outputFlow = controlInterface.readOutputFlow();

          return state;

	} // getSourceState

} // class StationMonitor

