package cz.zcu.fav.kiv.jsimcasestudies.watersystem.model;

import cz.zcu.fav.kiv.jsimcasestudies.watersystem.*;


/**
 *  Simple data model of HW interrupt controller device
 *  with a given number of interrupt levels.
 *  Control program binds its service routine by calling bindInterrupt().
 *  Environment model side issues interrupt by calling setInterrupt();
 *  Nested service of interrupts is not assumed, i.e. interrupt level
 *  has not here a meaning of interrupt priority.
 *
 *  @author St.R.
 *  @version 1.0 from 2004-04-30
 */


public class InterruptController implements InterruptControllerInterface
{

        public int nLevels;                    // number of interrupt levels
        InterruptEntryInterface [] iEVector;   // service routines addresses
 	boolean [] iMaskVector;                // interrupts mask

        /**
	 *  constructor
	 *
         *  @param nLevels Number of interrupt levels
         *  @param iniMask Initial setting of the mask
         *
	 */
	public InterruptController(int nLevels, boolean iniMask) {
          this.nLevels = nLevels;
          iEVector = new InterruptEntryInterface[nLevels];
          iMaskVector = new boolean [nLevels];
          for (int i=0; i<nLevels; i++) {
            iEVector [i] = null;
            iMaskVector[i] = iniMask;
          }
        }


        /**
	 *  Binds the address of an interrupt service routine to a level of interrupt.
         *  Used from the control program side.
	 *
         *  @param level   Interrupt level
         *  @param ie      Interrupt entry (service routine)
	 */
	public void bindInterrupt(int level, InterruptEntryInterface ie) {
          if ((level >= 0) && (level < nLevels)) {
            iEVector [level] = ie;
          }
        }


        /**
	 *  Set a value of the mask.
         *  Used from the control program side.
	 *
         *  @param level      Interrupt level
         *  @param maskValue  New value of the mask
	 */
	public void maskInterrupt(int level,  boolean maskValue) {
          if ((level >= 0) && (level < nLevels)) {
            iMaskVector[level] = maskValue;
          }
        }


        /**
	 *  Issues the interrupt.
         *  Used from the environment model side.
	 *
         *  @param level   Interrupt level
	 */
	public void setInterrupt(int level) {
          if ((iEVector[level] != null) && (level >= 0) && (level < nLevels) && !iMaskVector[level])
            iEVector[level].interruptService();

        }
	
} //

