package cz.zcu.fav.kiv.jsimcasestudies.watersystem.model;

import cz.zcu.fav.kiv.jsim.*;

/**
 *  Simple dynamic model of HW timer.
 *  This process is responsible for periodic time signals issued
 *    as interrupts to the control program part. 
 *
 *  @author St.R.
 *  @version 1.3 from 2004-05-03
 */

class TimerProcess extends JSimProcess
{
	/**
	 *  Multiplier of the model's base period. The product of this multiplier and the model's base
	 *  period will be used as a parameter to the hold() method.
	 */
	private static final double BASE_PERIOD_MULTIPLIER = 1.0;

	/* ------------------------------------------------------------------------------------------------ */

	/**
	 *  The model's base period of updates.
	 */
	private final double basePeriod;
	

        /**
	 *  Interrupt controller object.
	 */
	private InterruptController ic;
	

	/* ------------------------------------------------------------------------------------------------ */

	/**
	 *  Constructs a new process that simulates HW timer. 
	 *
	 *  @param simulation	The overall simulation.	
         *  @param ic           Interrupt object (flag) sets with any sensor change	 
	 *  @param basePeriod	The model's base period.
	 */
	protected TimerProcess(JSimSimulation simulation, InterruptController ic, double basePeriod)
          throws JSimSimulationAlreadyTerminatedException,                       JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super("Timer Process", simulation);		
                this.ic = ic;		
		this.basePeriod = basePeriod;
	} // constructor

	/**
	 *  Simulates periodical time interrupts
	 */
	protected void life()
	{
		try
		{			
			
                        double holdTime = basePeriod * BASE_PERIOD_MULTIPLIER;

                        while (true)
			{
				
				hold(holdTime);

                                ic.setInterrupt(0);
                                // message(getProcessName());
                               
			} // while true
		} // try
		catch (JSimSecurityException e1)
		{
			e1.printComment(System.err);
			e1.printStackTrace();
		} // catch
	} // life


} // class TimerProcess

