/*
 *  A simple Hello-World example. One process is created but not activated. Therefore, nothing will happen.
 *
 */


import cz.zcu.fav.kiv.jsim.*;

/*
 *  A simple main-program example
 */
public class HelloWorld1
{
 public static void main(String[] args)
 {
  JSimSimulation        simulation = null;
  HelloWorld1Process    process = null;

  try
  {
   /*
    *  First, we create a simulation and one process.
    */
   simulation = new JSimSimulation("Hello World 1 Simulation");
   /*
    *  If you want to know what happens inside J-Sim and your simulation,
    *  uncomment the following line -- implicite value is false.
    */
   //simulation.setTalking(true);

   process = new HelloWorld1Process("Hello World 1 Process", simulation);

   /*
    *  Now we run the simulation. But nothing happens. Why? We have forgotten to activate the process!
    */
   while (simulation.step() == true)
    ;
  } // try

  /*
   *  All three possible exceptions are caught here in one handler
   */
  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  } // catch

  /*
   *  It is a good practice to put the shutdown in the finally block. Then you can be sure
   *  that that the JVM will not "hang". (This wouldn't occur here.)
   */
  finally
  {
   /*
    *  Every simulation must be shut down at the end of the program, even if an error occurs.
    */
   simulation.shutdown();
  }
 } // main
} // HelloWorld1


/*
 *  A simple process example
 */
class HelloWorld1Process extends JSimProcess
{
 /*
  *  Note that you must ALWAYS write your own constructor -- JSimProcess' one has two parameters
  */
 public HelloWorld1Process(String name, JSimSimulation sim) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
 {
  super(name, sim);
 } // constructor

 /*
  *  This is the life of the process. You can do whatever you want here.
  */
 protected void life()
 {
  message("Hello! It is " + myParent.getCurrentTime() + " o'clock now.");
  message("Good - Bye!");
 } // life
} // class HelloWorld1Process

