/*
 *  A simple Hello-World example. One process is created and activated.
 *  It tries to print current time infinite number of times and then let other processes run while he is sleeping.
 *
 */


import cz.zcu.fav.kiv.jsim.*;

/*
 *  A simple main-program example
 */
public class HelloWorld4
{
 public static void main(String[] args)
 {
  JSimSimulation        simulation = null;
  HelloWorld4Process    process = null;
  int                   i;

  try
  {
   /*
    *  First, we create a simulation and one process.
    */
   simulation = new JSimSimulation("Hello World 4 Simulation");
   /*
    *  If you want to know what happens inside J-Sim and your simulation,
    *  uncomment the following line -- implicite value is false.
    */
   //simulation.setTalking(true);

   process = new HelloWorld4Process("Hello World 4 Process", simulation);

   /*
    *  We must not forget to activate the process. A nice time, isn't it?
    */
   process.activate(1.2345);

   /*
    *  Now we run the simulation. We will do ten (or less) steps. This is a more
    *  safe way how to run your simulation since you have full control over all processes.
    */
   for (i = 0; (i < 10) && (simulation.step() == true); i++)
    ;
  } // try

  /*
   *  All three possible exceptions are caught here in one handler
   */
  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  } // catch

  /*
   *  It is a good practice to put the shutdown in the finally block. Then you can be sure
   *  that that the JVM will not "hang".
   */
  finally
  {
   /*
    *  Every simulation must be shut down at the end of the program, even if an error occurs.
    */
   simulation.shutdown();
  }
 } // main
} // HelloWorld4


/*
 *  A simple process example. It prints current simulation time infinite number of times
 *  and therefore never dies.
 */
class HelloWorld4Process extends JSimProcess
{
 /*
  *  Note that you must ALWAYS write your own constructor -- JSimProcess' one has two parameters
  */
 public HelloWorld4Process(String name, JSimSimulation sim) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
 {
  super(name, sim);
 } // constructor

 /*
  *  This is the life of the process. We will cycle here infinite number of times.
  */
 protected void life()
 {
  try
  {
   message("Hello! I have just been born!");

   while (true)
   {
    message("Hello! It is " + myParent.getCurrentTime() + " o'clock now and I'm going to sleep for a while.");
    /*
     *  Now we let other processes run. That's a pity we are alone here!
     */
    hold(9.8765);
   } // for i

   /*
    *  We will never get here.
    */
   // message("It is not possible that you see this message.");
   // message("Good - Bye! My time is over.");
  } // try
  catch (JSimSecurityException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  } // catch
 } // life
} // class HelloWorld4Process

