/*
 *  Process creating four sons, activating them and then dying.
 */

import cz.zcu.fav.kiv.jsim.*;


class DynamicProcess extends JSimProcess
{
 /*
  *  Be very careful when changing this constant. Number of processes grows exponentially!
  *
  *  LEVEL_LIMIT        processes in simulation (not necessarly running in the same time)
  *  0                  1
  *  1                  5       (1 + 4)
  *  2                  21      (1 + 4 + 16)
  *  3                  83      (1 + 4 + 16 + 64)
  *  4                  339     (1 + 4 + 16 + 64 + 256)
  */
 private static final int       LEVEL_LIMIT = 4;
 private int                    myLevel;
 /*
  *  Note that you must ALWAYS write your own constructor -- JSimProcess' one has two parameters
  */
 public DynamicProcess(String name, JSimSimulation sim, int level) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
 {
  super(name, sim);
  myLevel = level;
 } // constructor

 /*
  *  This is our life, where we create the sons and run them.
  */
 protected void life()
 {
  try
  {
   DynamicProcess       sonA, sonB, sonC, sonD;
   double               time;

   /* Really create? */
   if (myLevel < LEVEL_LIMIT)
   {
    time = myParent.getCurrentTime();
    message("Time is " + time);
    message(getName() + ": I'm going to create four sons and run them.");
    sonA = new DynamicProcess(getName() + "A", myParent, myLevel+1);
    sonB = new DynamicProcess(getName() + "B", myParent, myLevel+1);
    sonC = new DynamicProcess(getName() + "C", myParent, myLevel+1);
    sonD = new DynamicProcess(getName() + "D", myParent, myLevel+1);
    sonA.activate(time + 0.03125);
    sonB.activate(time + 0.06250);
    sonC.activate(time + 0.09375);
    sonD.activate(time + 0.12500);

    /*
     *  Now we sleep for 2 seconds and we will wake up after our sons will have run.
     */
    message(getName() + ": I'm going to sleep for 2 seconds.");
    hold(2.0);
    time = myParent.getCurrentTime();
    message("Time is " + time);
    message(getName() + ": Good-bye.");
   }
   else // We don't create any new process
   {
    message("Time is " + myParent.getCurrentTime());
    message(getName() + ": I'm the last one in our family hierarchy. Good-bye.");
   } // else
  } // try
  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  }
 } // life
} // class DynamicProcess

