/*
 *  An example on creating dynamic processes. During his life, every process creates four sons
 *  and activates them. The sons have the same name as their father but with an extra letter (A, B, C or D)
 *  at the end. The level of recursion is limited by 4.
 *
 */


import cz.zcu.fav.kiv.jsim.*;

/*
 *  The main program just creates the first one and activates him.
 */
public class DynamicProcessesExample
{
 public static void main(String[] args)
 {
  JSimSimulation        simulation = null;
  DynamicProcess        dp = null;

  try
  {
   /*
    *  First, we create a simulation and one process.
    */
   simulation = new JSimSimulation("Dynamic Processes Example Simulation");
   /*
    *  If you want to know what happens inside J-Sim and your simulation,
    *  uncomment the following line -- implicite value is false.
    */
   // simulation.setTalking(true);

   /*
    *  This is the father of all other processes
    */
   dp = new DynamicProcess("X", simulation, 0);

   /*
    *  We must not forget to activate the process. It will create and activate the others.
    */
   dp.activate(0.0);

   /*
    *  Now we run the simulation until there are no processes scheduled.
    */
   while (simulation.step() == true)
    ;

   simulation.message("It's over.");
  } // try
  /*
   *  All three possible exceptions are caught here in one handler
   */
  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  } // catch

  /*
   *  It is a good practice to put the shutdown in the finally block. Then you can be sure
   *  that that the JVM will not "hang".
   */
  finally
  {
   /*
    *  Every simulation must be shut down at the end of the program, even if an error occurs.
    */
   simulation.shutdown();
  }
 } // main
} // DynamicProcessesExample

