/*
 *  A simple example on using queues. We insert seven items in the queue and then print them out.
 *
 */

import cz.zcu.fav.kiv.jsim.*;

public class QueueExample1
{
 public static void main(String[] args)
 {
  JSimSimulation        sim = null;
  JSimHead              queue;
  JSimLink              qi, after, before;
  int                   i;
  Integer               intObject;
  Float                 floatObject;
  String                dataType;

  try
  {
   /*
    *  It is necessary to create a simulation before creating a queue -- the queue needs simulation's time functions.
    */
   sim = new JSimSimulation("Simulation with queues");
   /*
    *  You don't need to set the talking on here.
    */
   // sim.setTalking(true);
   queue = new JSimHead("Little queue", sim);

   /*
    *  Let's insert some data into the queue...
    */
   for (i = 0; i < 5; i++)
   {
    qi = new JSimLink(new Integer(i));
    qi.into(queue);
   } // for

   sim.message("Number of items in the queue: " + queue.cardinal());
   sim.message("Is the queue empty? " + queue.empty());

   /*
    *  Now we insert two other items into the queue -- one after the first item and one before the last one.
    */
   after  = new JSimLink(new Float(0.5));
   before = new JSimLink(new Float(3.5));

   after.follow(queue.first());
   before.precede(queue.last());

   sim.message("Number of items in the queue: " + queue.cardinal());
   sim.message("Is the queue empty? " + queue.empty());

   /* 0 0.5 1 2 3 3.5 4 */

   /*
    *  Now, we will print out all the data stored in the queue, taking them out of the queue
    */
   while (!queue.empty())
   {
    qi = queue.first();
    dataType = qi.getDataType();
    sim.message("The first item is of type: " + dataType);
    switch (dataType.charAt(dataType.lastIndexOf('.') + 1))
    {
     case 'I':
     case 'i':
      intObject = (Integer) qi.getData();
      sim.message("Data: " + intObject.toString());
      break;
     case 'F':
     case 'f':
      floatObject = (Float) qi.getData();
      sim.message("Data: " + floatObject.toString());
      break;
     default:
      sim.message("There is an unknown object at the head of the queue.");
    } // switch
    /*
     *  Take him out!
     */
    qi.out();
    sim.message("Number of items in the queue: " + queue.cardinal());
   } // while
   sim.message("Clearing the queue...");
   queue.clear();

  } // try
  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  } // catch
  /*
   *  It is not necessary to shutdown the simulation when there are no processes,
   *  but maybe a process will be added later and you will forget to do it.
   */
  finally
  {
   sim.shutdown();
  } // finally
 } // main
} // class QueueExample1

