import cz.zcu.fav.kiv.jsim.*;

/*
 *  Periodicly gets data from the queue.
 *
 */

public class ConsumerProcess extends JSimProcess
{
 private JSimHead       queue;
 private double         lambda;
 /*
  *  It is better to throw the exceptions from the constructor than to catch them
  */
 public ConsumerProcess(String name, JSimSimulation simulation, JSimHead q, double l) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
 {
  super(name, simulation);
  if (q == null)
   throw new JSimInvalidParametersException("ConsumerProcess.ConsumerProcess(): q");
  queue = q;
  lambda = l;
 } // constructor

 /*
  *  We don't care about stopping -- we run infinitely
  */
 protected void life()
 {
  try
  {
   JSimLink     link;
   double       time;
   Integer      intObject;

   /*
    *  Yes, it's really a neverending cycle
    */
   while (true)
   {
    time = myParent.getCurrentTime();
    /*
     *  If the queue is empty, we don't do anything.
     */
    if (queue.empty())
     message(time + " -- " + getName() + ": queue empty.");
    else
    {
     /*
      *  We must pick up the first item and take it out from the queue.
      */
     link = queue.first();
     link.out();
     intObject = (Integer) link.getData();
     message(time + " -- " + getName() + ": received " + intObject.toString());
    }

    hold(JSimSystem.negExp(lambda));
   } // while
  } // try
  catch (JSimSecurityException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  }
 } // life
} // class ConsumerProcess

