/*
 *  A simple example using GUI output.
 *  Two processes are created and the first one is activated, but no step is done.
 *  Instead we switch to graphic mode where the user can control the simulation using the mouse.
 */
import cz.zcu.fav.kiv.jsim.*;

public class ActivatePassivate
{
 public static void main(String[] args)
 {
  JSimSimulation        simulation = null;
  ActivatePassivateProcess    processA = null, processB = null;

  try
   {
   /*
    *  As usual, we must create a simulation first.
    */
   System.out.println("Starting up simulation...");
   simulation = new JSimSimulation("Activate Passivate GUI Simulation", JSimSimulation.MODE_GUI_INTERACTIVE);

   /*
    *  Uncomment the following line, if you want to see all details.
    */
   //simulation.setTalking(true);

   /*
    *  Now creating and activating processes.
    */
   simulation.message("Creating two processes...");
   processA = new ActivatePassivateProcess("Hello!", simulation, 0.5);
   processB = new ActivatePassivateProcess("Ahoj!", simulation, 0.6);

   /*
    *  We must set the neighbours now because they didn't exist at the time the processes were created.
    */
   processA.setNeighbour(processB);
   processB.setNeighbour(processA);

   /*
    * Activating only processA
    */
   processA.activate(0.0);

   /*
    *  Now we switch to graphic mode and let user do whatever he wants.
    */
   simulation.runGUI();

  }

  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  }

  /*
   *  As usual, don't forget to shutdown the simulation. Do it always in the finally block.
   */
 finally
  {
   simulation.shutdown();
  }
 } // main
} // class ActivatePassivate



/*
 *  An example of simple process. Activates it's neigbour and then passivates itself.
 *  This process could be use in a console-output simulation too, there is no difference.
 */

class ActivatePassivateProcess extends JSimProcess
{
 private String         helloWord;
 private double         lambda;
 private JSimProcess    neighbour;

 public ActivatePassivateProcess(String hello, JSimSimulation sim, double l) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
 {
  super("Process that says " + hello, sim);
  helloWord = hello;
  lambda = l;
 } // constructor

 /*
  *  Note that we don't care at all about the mode we run in. We don't even know about it!
  */
 protected void life()
 {
  double        timeToActivate;

  try
  {
   /* To die or to live, that's the question. */
   while (true)
   {
    message(helloWord + " I'm going to activate my neighbour " + neighbour.getName() + ".");
    timeToActivate = JSimSystem.negExp(lambda);
    neighbour.activate(myParent.getCurrentTime() +   timeToActivate);
    message(helloWord + " I'm going to passivate myself.");
    passivate();
   } // while
  }
  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  }
 } // life

 public void setNeighbour(JSimProcess proc)
 {
  neighbour = proc;
 }
} // class ActivatePassivateProcess
