/*
 *  A simple example using GUI output.
 *  A simplified producer - consumer example. Two processes share the same queue,
 *  one puts data into it and the other retrieves them.
 */

import cz.zcu.fav.kiv.jsim.*;

public class ProducerConsumerExample
{
 public static void main(String[] args)
 {
  JSimSimulation        sim = null;
  JSimHead              queue = null;
  ProducerProcess       producer = null;
  ConsumerProcess       consumer = null;
  int                   i;

  try
  {
   /*
    *  It is necessary to create a simulation before creating a queue -- the queue needs simulation's time functions.
    */
   sim = new JSimSimulation("Producer-Consumer Simulation", JSimSimulation.MODE_GUI_INTERACTIVE);
   /*
    *  You don't need to set the talking on here.
    */
   // sim.setTalking(true);
   queue = new JSimHead("Shared Queue", sim);

   /*
    *  Let's create the processes.
    */
   producer = new ProducerProcess("Producer", sim, queue, 0.25);
   consumer = new ConsumerProcess("Consumer", sim, queue, 0.25);

   /*
    * Consumer will be started first, but he will switch to producer.
    */
   producer.activate(1.0);
   consumer.activate(0.0);

   /*
    *  Now we switch to graphic mode and let user do whatever he wants.
    *  This is endless simulaton, it can be terminated only by pressing the quit button.
    */ 

   sim.runGUI();
     
   sim.message("Bye!");
  } // try
  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  }

  /*
   *  We must not forget to shut the simulation down!
   */
  finally
  {
   sim.shutdown();
  }
 } // main
} // class ProducerConsumerExample
