/*
 *  Periodically takes transactions from its queue and inserts them in another queue or deletes them.
 *
 */

import cz.zcu.fav.kiv.jsim.*;

public class Server extends JSimProcess
{
 private double                 mi;
 private double                 p;
 private QueueWithServer        queueIn;
 private QueueWithServer        queueOut;

 private int                    counter;
 private double                 transTq;


 public Server(String name, JSimSimulation sim, double parMi, double parP, QueueWithServer parQueueIn, QueueWithServer parQueueOut) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
 {
  super(name, sim);
  mi = parMi;
  p = parP;
  queueIn = parQueueIn;
  queueOut = parQueueOut;

  counter = 0;
  transTq = 0.0;
 } // constructor

 protected void life()
 {
  Transaction   t;
  JSimLink      link;

  try
  {
   while (true)
   {
    if (queueIn.empty())
    {
     /*
      *  If we have nothing to do, we sleep.
      */
     //message(getProcessName() + ": The queue is empty, going to sleep.");
     passivate();
     //message(getProcessName() + ": Something has happened with my queue, woken up.");
    }
    else
    {
     /*
      *  Simulating hard work here...
      */
     hold(JSimSystem.negExp(mi));
     //message(getProcessName() + ": Taking a transaction from the queue.");
     link = queueIn.first();

     /*
      *  Now we must decide whether to throw the transaction away or to insert it into another queue
      */
     if (JSimSystem.uniform(0.0, 1.0) > p)
     {
      //message(getProcessName() + ": Throwing the transaction away.");
      t = (Transaction) link.getData();
      counter++;
      transTq += myParent.getCurrentTime() - t.getCreationTime();
      link.out();
      link = null;
     }
     else
     {
      //message(getProcessName() + ": Inserting the transaction into another queue.");
      link.out();
      link.into(queueOut);
      if (queueOut.getServer().isIdle())
      {
       //message(getProcessName() + ": The other queue's server is idle, waking it up.");
       queueOut.getServer().activate(myParent.getCurrentTime());
      } // if isIdle
     } // else throw away / insert
    } // else queue is empty / not empty
   } // while
  } // try
  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  }
 } // life

 public int getCounter()
 {
  return counter;
 } // getCounter

 public double getTransTq()
 {
  return transTq;
 } // getTransTq
} // class Server

