/*
 *  An example of a neverending simulation running in graphic batch mode.
 *  The execution can be interrupted with the `Quit' button only.
 */

import cz.zcu.fav.kiv.jsim.*;

public class ActivatePassivate
{
 public static void main(String[] args)
 {
  JSimSimulation        simulation = null;
  ActivatePassivateProcess    processA = null, processB = null;

  try
   {
   /*
    *  As usual, we must create a simulation first.
    */
   System.out.println("Starting up simulation...");
   simulation = new JSimSimulation("Activate Passivate Simulation", JSimSimulation.MODE_GUI_BATCH);

   /*
    *  Uncomment the following line, if you want to see all details.
    */
   //simulation.setTalking(true);

   /*
    *  Now creating and activating processes.
    */
   simulation.message("Creating two processes...");
   processA = new ActivatePassivateProcess("Hello!", simulation, 0.5);
   processB = new ActivatePassivateProcess("Ahoj!", simulation, 0.6);

   /*
    *  We must set the neighbours now because they didn't exist at the time the processes were created.
    */
   processA.setNeighbour(processB);
   processB.setNeighbour(processA);

   /*
    * Activating only processA
    */
   processA.activate(0.0);

   /* This is an endless simulaton, it can be terminated only by pressing the quit button */
   while (simulation.step() == true)
    ;

   /*
    *  This suspends the program until you close the main window
    *  without this you will not see any results after simulation's termination
    *  but this simulation never ends so this command does not need to be here
    */
   //simulation.waitForWindowClosure();
  } // try

  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  }

  /*
   *  As usual, don't forget to shutdown the simulation. Do it always in the finally block.
   */
 finally
  {
   simulation.shutdown();
  }
 } // main
} // class ActivatePassivate



/*
 *  An example of simple process. Activates it's neigbour and then passivates itself.
 *  This process could be use in a console-output simulation too, there is no difference.
 */
class ActivatePassivateProcess extends JSimProcess
{
 private String         helloWord;
 private double         lambda;
 private JSimProcess    neighbour;

 public ActivatePassivateProcess(String hello, JSimSimulation sim, double l) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
 {
  super("Process that says " + hello, sim);
  helloWord = hello;
  lambda = l;
 } // constructor

 /*
  *  Note that we don't care at all about the mode we run in. We don't even know about it!
  */
 protected void life()
 {
  double        timeToActivate;

  try
  {

   while (true)
   {
    message(helloWord + " I'm going to activate my neighbour " + neighbour.getName() + ".");
    timeToActivate = JSimSystem.negExp(lambda);
    neighbour.activate(myParent.getCurrentTime() +   timeToActivate);
    message(helloWord + " I'm going to passivate myself.");
    passivate();
   } // while
  }
  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  }
 } // life


 public void setNeighbour(JSimProcess proc)
 {
  neighbour = proc;
 }
} // class ActivatePassivateProcess
