/*
 *  Another example of batch GUI mode simulation. Processes printing out greetings
 *  are infinite, i.e. they cycle in a neverending loop.
 *  When step() or message() is called for the first time, simulation switches to graphic mode.
 */

import cz.zcu.fav.kiv.jsim.*;

public class GUIGreetings
{
 public static void main(String[] args)
 {
  JSimSimulation        simulation = null;
  GUIGreetingsProcess   processA = null, processB = null, processC = null, processD = null, processE = null;

  try
   {
   /*
    *  As usual, we must create a simulation first.
    */
   System.out.println("Starting up simulation...");
   simulation = new JSimSimulation("Hello GUI World Simulation", JSimSimulation.MODE_GUI_BATCH);

   /*
    *  Uncomment the following line, if you want to see all details.
    */
   //simulation.setTalking(true);

   /*
    *  Now creating and activating processes in exactly the same way as we do it using console output.
    */
   simulation.message("Creating five processes...");
   processA = new GUIGreetingsProcess("Hello!", simulation, 0.5);
   processB = new GUIGreetingsProcess("Ciao!" , simulation, 0.6);
   processC = new GUIGreetingsProcess("Salut!", simulation, 1.5);
   processD = new GUIGreetingsProcess("Hej!"  , simulation, 1.0);
   processE = new GUIGreetingsProcess("Ahoj!" , simulation, 1.2);

   processA.activate(0.0);
   processB.activate(0.5);
   processC.activate(1.0);
   processD.activate(1.2);
   processE.activate(1.5);

   // this is endless simulaton, it can be terminated only by pressing then quit button
   while (simulation.step() == true);
    ;
   
   /*
    *  This suspends the program until you close the main window
    *  without this you will not see any results after simulation's termination
    *  but this simulation never ends so this command does not need to be here
    */
   simulation.waitForWindowClosure();
  
   simulation.talk("Back on the console, good-bye.");
  }

  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  }

  /*
   *  As usual, don't forget to shutdown the simulation. Do it always in the finally block.
   */
  finally
  {
   simulation.shutdown();
  }
 } // main
} // class GUIGreetings



/*
 *  An example of simple process. Writes "Hello" or its equivalent and then sleeps.
 *  This process could be use in a console-output simulation too, there is no difference.
 */

class GUIGreetingsProcess extends JSimProcess
{
 private String         helloWord;
 private double         lambda;

 public GUIGreetingsProcess(String hello, JSimSimulation sim, double l) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
 {
  super("Process that says " + hello, sim);
  helloWord = hello;
  lambda = l;
 } // constructor

 /*
  *  Note that we don't care at all about the mode we run in. We don't even know about it!
  */
 protected void life()
 {
  double        timeToSleep;

  try
  {
   /*  Infinite process */
   while (true)
   {
    timeToSleep = JSimSystem.negExp(lambda);
    message(helloWord + " I'm going to sleep for " + timeToSleep + " seconds.");
    hold(timeToSleep);
   } // while
  }
  catch (JSimSecurityException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  }
 } // life
} // class GUIGreetingsProcess
