/*
 *  A queue having its own server which takes transactions out from the queue.
 *
 *  Methods getObjectListItemDescription() and getDetailedInformationArray()
 *  are overridden with our own code which modifies the set of characteristics usually shown.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.gui.*;

import java.util.*;

public class QueueWithServer extends JSimHead
{
 private JSimProcess         server;

 public QueueWithServer(String name, JSimSimulation sim, JSimProcess s) throws JSimInvalidParametersException, JSimTooManyHeadsException
 {
  super(name, sim);
  server = s;
 } // constructor

 public JSimProcess getServer()
 {
  return server;
 } // getServer

 public void setServer(JSimProcess s)
 {
  server = s;
 } // setServer


 /*
  *  Overrides the method from class JSimHead.
  *  Now the detailed info window displays the average length of the queue
  *  in addition to the standard information.
  */
 public String getObjectListItemDescription()
 {
  return("No.: " + getNumber() + "  Name: " + getName() + "  Elements: " + cardinal() + "  Average length: " + getLw());
 }


 /*
  *  Overriding the method from class JSimHead.
  *  A new row is added to the table: the average waiting time of all elements in the queue.
  */
 public Collection getDetailedInformationArray()
 {
  Collection c;
  c = (java.util.List) super.getDetailedInformationArray();
  c.add(new JSimPair("Average waiting time for all links:", Double.toString(getTwForAllLinks())));
  return(c);
 }
} // class QueueWithServer

