/*
 *  A simple generator. Creates new transactions and puts them into a queue.
 *  Look into the documentation for additional information.
 *  Methods getObjectListItemDescription() and getDetailedInformationArray() are overridden
 *  to provide some extra information.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.gui.*;
import java.util.*;

public class Generator extends JSimProcess
{
 private double                 lambda;
 private QueueWithServer        queue;
 private long                   transactionsCreated;


 public Generator(String name, JSimSimulation sim, double l, QueueWithServer q) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
 {
  super(name, sim);
  lambda = l;
  queue = q;
  transactionsCreated = 0;
 } // constructor

 protected void life()
 {
  JSimLink      link;

  try
  {
   while (true)
   {
    /*
     *  Periodically creating new transactions and putting them into the queue.
     */
    //message(getProcessName() + ": A new transaction created.");
    link = new JSimLink(new Transaction(myParent.getCurrentTime()));
    link.into(queue);
    transactionsCreated++;
    if (queue.getServer().isIdle())
    {
     //message(getProcessName() + ": The server is idle, waking it up...");
     queue.getServer().activate(myParent.getCurrentTime());
    }
    hold(JSimSystem.negExp(lambda));
   } // while
  } // try
  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  }
 } // life

 /*
  *  Overrides the method from class JSimProcess.
  */
 public String getObjectListItemDescription()
 {
  return(super.getObjectListItemDescription() + "  Transactions created: " + transactionsCreated);
 }

 /*
  *  Overrides the method from class JSimProcess.
  *  Adds a new row to the table.
  */
 public Collection getDetailedInformationArray()
 {
  Collection c;
  c = (List) super.getDetailedInformationArray();
  c.add(new JSimPair("Transactions created:", Long.toString(transactionsCreated)));
  return c;
 }
} // class Generator

