/*
 *  If we dont't want to use the default detailed info window that J-Sim offers,
 *  it's possible to create our own window inherited from JSimGUIDetailedInfoWindow
 *  and to show whatever we want in this window.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.gui.*;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.event.*;
import java.util.*;


/*
 *  The MyGUIDetailedInfoWindow class provides a dialog showing a panel with a text.
 */
public class MyGUIDetailedInfoWindow extends JSimDetailedInfoWindow
{
 // Panel showing our own characteristics
 class TextPanel extends JPanel
 {
   public TextPanel()
   {
    setPreferredSize(new Dimension(200, 100));
    Border raisedBevel = BorderFactory.createRaisedBevelBorder();
    Border loweredBevel = BorderFactory.createLoweredBevelBorder();
    Border compound = BorderFactory.createCompoundBorder(raisedBevel, loweredBevel);
    setBorder(compound);
    setFont(new Font("SansSerif", Font.BOLD, 30));
   }

   // In this panel we show the number of elements in the queue (in graphic mode)
   public void paintComponent(Graphics g)
   {
    super.paintComponent(g);
    g.drawString(Long.toString(((JSimHead) objectInfo).cardinal()), getWidth()/2 - 10, getHeight()/2);
   }
 } // class TextPanel


 TextPanel panelSpec;

 // Just use parent constructor
 public MyGUIDetailedInfoWindow(JSimMainWindow parent, JSimHead info)
 {
  super(parent, info);
 }

 /*
  *  Overriding the method of its parent.
  *  This is the key action you have to do when replacing the content of the info window!
  */
 protected void constructorSpecificInit()
 {
  panelSpec = new TextPanel();
  getContentPane().add(new JLabel("Number of elements:"), BorderLayout.NORTH);
  getContentPane().add(panelSpec, BorderLayout.CENTER);
 }

 /*
  *  Overriding the method of its parent.
  */
 public void update(Observable o, Object arg)
 {
  panelSpec.repaint();
 } // update
} // MyGUIDetailedInfoWindow

