/*
 *  A queue having its own server which takes transactions out from the queue.
 *  This class overrides the createDetailedInfoWindow() method in order to replace the default
 *  window with its own one.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.gui.*;

import java.util.*;
import javax.swing.*;

public class QueueWithServer extends JSimHead
{
 private JSimProcess         server;

 public QueueWithServer(String name, JSimSimulation sim, JSimProcess s) throws JSimInvalidParametersException, JSimTooManyHeadsException
 {
  super(name, sim);
  server = s;
 } // constructor

 public JSimProcess getServer()
 {
  return server;
 } // getServer

 public void setServer(JSimProcess s)
 {
  server = s;
 } // setServer


 /*
  *  Overrides the method from class JSimHead.
  */
 public String getObjectListItemDescription()
 {
  return("No.: " + getNumber() + "  Name: " + getName() + "  Elements: " + cardinal() + "  Average length: " + getLw());
 }


 /*
  *  Overriding the method from class JSimHead.
  */
 public Collection getDetailedInformationArray()
 {
  Collection c;
  c = (java.util.List) super.getDetailedInformationArray();
  c.add(new JSimPair("Average waiting time for all links:", Double.toString(getTwForAllLinks())));
  return c;
 }


 /*
  *  Overrides the method from class JSimHead.
  *  Creates a window that shows completely different information from the default one.
  *  Always create your detailed info window this way!
  */
 public JDialog createDetailedInfoWindow(JSimMainWindow parentWindow)
 {
  MyGUIDetailedInfoWindow dWindow = new MyGUIDetailedInfoWindow(parentWindow, this);
  return dWindow;
 }
} // class QueueWithServer

