/*
 *  This class is a replacement for the default J-Sim info window. It does not only replaces the information
 *  content of the window but the whole window.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.gui.*;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
 *  MyGUIDetailedInfoWindow class provides a dialog which shows a panel with text
 */
public class MyGUIDetailedInfoWindow extends JDialog implements Observer
{
 private JSimMainWindow myParent;
 private JPanel  panel;
 private TextPanel panelSpec;
 private JButton button;
 private JSimProcess objectInfo;
 private JScrollPane panelSpecScrollPane;


 /*
  *  Inner class -- dialog's window adapter
  *
  *  Although we could implement it directly in the main window we rather create an extra class.
  *  We must implement ActionListener since there is no ActionAdapter to be possibly extended.
  *  We must not forget to remove object (whose information we display here) from hashtable
  *  in JSimGUIMainWindowSwing
  */
 class JSimGUIDetailedInfoWindowWindowAdapter extends WindowAdapter
 {
  public void windowClosing(WindowEvent e)
  {
   myParent.removeDisplayableFromOpenInfoWindows(objectInfo);
   setVisible(false);
   dispose();
  } //windowClosing
 } // class JSimGUIDetailedInfoWindowWindowAdapter


 /*
  *  Inner class -- dialog's action adapter
  *
  *  Although we could implement it directly in the main window we rather create an extra class.
  *  We must implement ActionListener since there is no ActionAdapter to be possibly extended.
  *  We must not to forget to remove object (whose information we display here) from hashtable
  *  in JSimGUIMainWindowSwing
  */
 class JSimGUIDetailedInfoWindowActionAdapter implements ActionListener
 {
  public void actionPerformed(ActionEvent e)
  {
   myParent.removeDisplayableFromOpenInfoWindows(objectInfo);
   setVisible(false);
   dispose();
  } // actionPerformed
 } // class JSimGUIDetailedInfoWindowActionAdapter


 // panel showing our own charakteristics
 class TextPanel extends JPanel {

  public TextPanel() {
   setPreferredSize(new Dimension(350, 100));
   Border raisedBevel = BorderFactory.createRaisedBevelBorder();
   Border loweredBevel = BorderFactory.createLoweredBevelBorder();
   Border compound = BorderFactory.createCompoundBorder(raisedBevel, loweredBevel);
   setBorder(compound);
   setFont(new Font("SansSerif", Font.BOLD, 20));
  }

  // In this panel we show process state (in graphics)
  public void paintComponent(Graphics g) {
   super.paintComponent(g);
   if (objectInfo.getState() == JSimProcess.STATE_SCHEDULED)
    g.drawString(objectInfo.getStateAsString() + " for " + objectInfo.getScheduleTime(), 10, getHeight()/2);
   else
    g.drawString(objectInfo.getStateAsString(), 10, getHeight()/2);

  }
 } // class TextPanel


 /*
  *  Creates a new dialog containing a panel (showing detailed information of object) and an OK button.
  *
  *  param parent parent window of the dialog
  *  param info process whose informations are about to be displayed in the panel
  */
 public MyGUIDetailedInfoWindow(JSimMainWindow parent, JSimProcess info)
 {
  super(parent, "My own window", false);
  myParent = parent;
  objectInfo = info;
  panel = new JPanel();
  button = new JButton("OK");
  panel.add(button);
  panelSpec = new TextPanel();
  panelSpecScrollPane = new JScrollPane(panelSpec);

  getContentPane().add(new JLabel("Process state:"), "North");
  getContentPane().add(panelSpecScrollPane, "Center");
  getContentPane().add(panel, "South");

  button.addActionListener(new JSimGUIDetailedInfoWindowActionAdapter());
  addWindowListener(new JSimGUIDetailedInfoWindowWindowAdapter());
  /* we must call update to fill the table model */
  Observable a = null;
  Object b = null;
  update(a, b);

  pack();
  /*
   *  It's better to have the dialog centered.
   */
  Toolkit toolkit = Toolkit.getDefaultToolkit();
  Dimension dim = toolkit.getScreenSize();
  Rectangle rect = getBounds();
  setLocation(dim.width/2 - rect.width/2, dim.height/2 - rect.height/2);
 } // constructor


 /*
  *  Updates the table model.
  */
 public void update(Observable o, Object arg)
 {
  panelSpec.repaint();
 } // update

} // MyGUIDetailedInfoWindow
