/*
 *  The aim of this program is to find the maximum number of J-Sim processes that can be created on your system.
 *  This number depends on your hardware, OS and Java runtime environment.
 */


import cz.zcu.fav.kiv.jsim.*;

/*
 *  A simple main-program example
 */
public class MaxNumberTesting
{
 public static void main(String[] args)
 {
  JSimSimulation        simulation = null;
  TestingProcess	process = null;
  int                   i = 0;
  int			max = 0;

  try
  {
   System.out.println("---------------------------------");
   System.out.println("Testing " + JSimSystem.getVersion());
   System.out.println("---------------------------------");
   System.getProperties().list(System.out);
   System.out.println("---------------------------------");
   /*
    *  First, we create a simulation and processes.
    */
   simulation = new JSimSimulation("Max Number of Processes Testing Simulation");
   /*
    *  If you want to know what happens inside J-Sim and your simulation,
    *  uncomment the following line -- implicite value is false.
    */
   //simulation.setTalking(true);

   try
   {
   	process = new TestingProcess("Process #" + Integer.toString(i), simulation);
   	process.activate(0.0);
   	i = 1;
   	while (simulation.step() == true)
   	{
	   	process = new TestingProcess("Process #" + Integer.toString(i), simulation);
	   	process.activate(simulation.getCurrentTime() + 0.01);
   		i++;
   	} // while
   } // try
   catch (OutOfMemoryError err)
   {
   	max = i-1;
   	//err.printStackTrace();
   } // catch

   System.out.println("Your system is able to create " + Integer.toString(max) + " J-Sim processes at this moment.");
  } // try

  /*
   *  All possible exceptions are caught here in one handler
   */
  catch (JSimException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  } // catch

  /*
   *  It is a good practice to put the shutdown in the finally block. Then you can be sure
   *  that that the JVM will not "hang".
   */
  finally
  {
   /*
    *  Every simulation must be shut down at the end of the program, even if an error occurs.
    */
   simulation.shutdown();
  }
 } // main
} // PerformanceTesting


/*
 *  A process does some `useful' work and then suspends itself for a given fixed time.
 */
class TestingProcess extends JSimProcess
{
 /*
  *  Note that you must ALWAYS write your own constructor -- JSimProcess' one has two parameters
  */
 public TestingProcess(String name, JSimSimulation sim) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
 {
  super(name, sim);
 } // constructor

 /*
  *  This is the life of the process. We will cycle here infinite number of times, doing some simple calculations.
  */
 protected void life()
 {
  int   i;

  try
  {
   while (true)
   {
    i = 1+1;
    hold(2.0);
   } // for i
  } // try
  catch (JSimSecurityException e)
  {
   e.printStackTrace();
   e.printComment(System.err);
  } // catch
 } // life
} // class MaxTesting

