/*
 *  A process performs two main activities in an infinite loop: local computation and update
 *  of a global result protected by a critical section. The activities of both parts are
 *  simulated by using hold().
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class ProcessWithSemaphoreAndData extends JSimProcess
{
	private double mu;
	private double lambda;

	private JSimSemaphore semaphore;
	private SharedData data;

//--------------------------------------------------------------------------------------------------

	public ProcessWithSemaphoreAndData(String name, JSimSimulation parent, double mu, double lambda, JSimSemaphore semaphore, SharedData data) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		this.mu = mu;
		this.lambda = lambda;
		this.semaphore = semaphore;
		this.data = data;
	} // constructor

	protected void life()
	{
		try
		{
			double myResult;

			while (true)
			{
				// Local part
				myResult = JSimSystem.negExp(0.1);
				hold(JSimSystem.negExp(lambda));

				// Critical section
				semaphore.P();
				data.setResult(0.99*data.getResult() + 0.01*myResult);
				data.incrementCountOfCriticalSectionUsage();
				hold(JSimSystem.negExp(mu));
				semaphore.V();
			} // while
		} // try
		catch (JSimException e)
		{
			e.printComment(System.err);
			e.printStackTrace();
		} // catch
	} // life

} // class ProcessWithSemaphoreAndData

