/*
 *	A simple process periodically retrieving messages from a shared message box.
 *	If no message is read from the message box, null is returned and the proces goes on.
 *	After every reading, the process sleeps for a while.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class ReceivingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

	private JSimMessageBox box;

//--------------------------------------------------------------------------------------------------

	public ReceivingProcess(String name, JSimSimulation parent, JSimMessageBox messageBox) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		box = messageBox;
	} // constructor

	protected void life()
	{
		JSimMessage message;
		double sleepTime;

		try
		{
			while (true)
			{
				message(myParent.getCurrentTime() + " - " + getName() + ": Trying to receive some data from the shared message box.");
				message = receiveMessageWithoutBlocking(box);
				if (message == null)
					message(myParent.getCurrentTime() + " - " + getName() + ": The box is empty.");
				else
					message(myParent.getCurrentTime() + " - " + getName() + ": Received data: " + message.getData().toString());

				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class ReceivingProcess

