/*
 *	A simple simulation of two processes exchanging messages asynchronously and indirectly via two message boxes.
 *	The messages know neither the sender nor the receiver. Neither the sending nor the receiving operation
 *	blocks. If no message is found in the message box by the receiving process, the process just goes on.
 *	The sender randomly chooses a message box to send the message to. So does the receiver.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class AsynchronousIndirectCommunicationSimulation
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		JSimMessageBox box1 = null;
		JSimMessageBox box2 = null;
		SendingProcess sender;
		ReceivingProcess receiver;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Asynchronous Indirect Communication Simulation");
			box1 = new JSimMessageBox("Shared Message Box 1");
			box2 = new JSimMessageBox("Shared Message Box 2");
			sender = new SendingProcess("Sender", simulation, box1, box2);
			receiver = new ReceivingProcess("Receiver", simulation, box1, box2);

			simulation.message("Activating the processes...");
			sender.activate(0.0);
			receiver.activate(0.0);

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch

		// We must shutdown the simulation.
		finally
		{
			simulation.shutdown();
		}
	} // main

} // class AsynchronousIndirectCommunicationSimulation

