/*
 *	A simple process periodically retrieving messages from one of two message boxes.
 *	If no message is read from the message box, null is returned and the proces goes on.
 *	After every reading, the process sleeps for a while.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class ReceivingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

	private JSimMessageBox box1;
	private JSimMessageBox box2;

//--------------------------------------------------------------------------------------------------

	public ReceivingProcess(String name, JSimSimulation parent, JSimMessageBox messageBox1, JSimMessageBox messageBox2) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		box1 = messageBox1;
		box2 = messageBox2;
	} // constructor

	protected void life()
	{
		JSimMessage message;
		double sleepTime;
		JSimMessageBox selectedBox;

		try
		{
			while (true)
			{
				if (JSimSystem.draw(0.5))
					selectedBox = box1;
				else
					selectedBox = box2;
				message(myParent.getCurrentTime() + " - " + getName() + ": Trying to receive some data from box `" + selectedBox.getName() + "'.");
				message = receiveMessageWithoutBlocking(selectedBox);
				if (message == null)
					message(myParent.getCurrentTime() + " - " + getName() + ": The box is empty.");
				else
					message(myParent.getCurrentTime() + " - " + getName() + ": Received data: " + message.getData().toString());

				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class ReceivingProcess

