/*
 *	A simple process periodically inserting messages to one of two shared message boxes. After every insertion,
 *	the process sleeps for a while.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class SendingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

	private JSimMessageBox box1;
	private JSimMessageBox box2;

//--------------------------------------------------------------------------------------------------

	public SendingProcess(String name, JSimSimulation parent, JSimMessageBox messageBox1, JSimMessageBox messageBox2) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		box1 = messageBox1;
		box2 = messageBox2;
	} // constructor

	protected void life()
	{
		JSimIndirectMessage message;
		int counter = 0;
		double sleepTime;
		JSimMessageBox selectedBox;

		try
		{
			while (true)
			{
				message = new JSimIndirectMessage(new Integer(counter));
				if (JSimSystem.draw(0.5))
					selectedBox = box1;
				else
					selectedBox = box2;
				message(myParent.getCurrentTime() + " - " + getName() + ": Sending a message with data `" + counter + "' to box `" + selectedBox.getName() + "'.");
				sendMessageWithoutBlocking(message, selectedBox);
				message(myParent.getCurrentTime() + " - " + getName() + ": Mesage sent.");
				counter++;
				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class SendingProcess

