/*
 *	A simple simulation of tree processes exchanging messages asynchronously and directly.
 *	The messages know both the sender and the receiver. Neither the sending nor the receiving operation
 *	blocks. If no message is found in the message box by the receiving process, the process just goes on.
 *	The receiving process tries to read messages from both senders but one of them is never activated
 *	so the receiver will be successful in half cases only.
 *
 *	The receiver decides from which process the new message will be received.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class AsynchronousSymmetricCommunicationSimulation
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		SendingProcess sender;
		SendingProcess passiveSender;
		ReceivingProcess receiver;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Asynchronous Symmetric Communication Simulation");
			sender = new SendingProcess("Sender", simulation);
			passiveSender = new SendingProcess("Passive Sender", simulation);
			receiver = new ReceivingProcess("Receiver", simulation, sender, passiveSender);
			sender.setReceiver(receiver);

			simulation.message("Activating the processes...");
			sender.activate(0.0);
			receiver.activate(0.0);
			// Passive receiver is not activated, it will never send a message.

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch

		// We must shutdown the simulation.
		finally
		{
			simulation.shutdown();
		}
	} // main

} // class AsynchronousSymmetricCommunicationSimulation

