/*
 *	A simple process periodically reading messages from two other processes.
 *	If no message is available from the selected process, null is returned and the receiving process goes on.
 *	After every reading, the process sleeps for a while.
 *
 *	The receiver decides from which process the new message will be received.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class ReceivingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

	private JSimProcess sender1;
	private JSimProcess sender2;

//--------------------------------------------------------------------------------------------------

	public ReceivingProcess(String name, JSimSimulation parent, JSimProcess sendingProcess1, JSimProcess sendingProcess2) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		sender1 = sendingProcess1;
		sender2 = sendingProcess2;
	} // constructor

	protected void life()
	{
		JSimMessage message;
		double sleepTime;
		JSimProcess selectedProcess;

		try
		{
			while (true)
			{
				if (JSimSystem.draw(0.5))
					selectedProcess = sender1;
				else
					selectedProcess = sender2;
				message(myParent.getCurrentTime() + " - " + getName() + ": Trying to receive some data from process `" + selectedProcess.getName() + "'.");
				message = receiveMessageWithoutBlocking(selectedProcess);
				if (message == null)
					message(myParent.getCurrentTime() + " - " + getName() + ": The selected process has not sent anything.");
				else
					message(myParent.getCurrentTime() + " - " + getName() + ": Received data: " + message.getData().toString() + ", sender is `" + message.getSender().getName() + "'.");

				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class ReceivingProcess

