/*
 *	A simple process periodically sending symmetric messages to a receiving process. After every send,
 *	the process sleeps for a while. The receiver must be set before the process is run.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class SendingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

	private JSimProcess receiver;

//--------------------------------------------------------------------------------------------------

	public SendingProcess(String name, JSimSimulation parent) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		receiver = null;
	} // constructor

	protected void life()
	{
		JSimSymmetricMessage message;
		int counter = 0;
		double sleepTime;

		try
		{
			while (true)
			{
				message = new JSimSymmetricMessage(this, receiver, new Integer(counter));
				message(myParent.getCurrentTime() + " - " + getName() + ": Sending a message with data `" + counter + "' to process `" + receiver.getName() + "'.");
				sendMessageWithoutBlocking(message);
				message(myParent.getCurrentTime() + " - " + getName() + ": Mesage sent.");
				counter++;
				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

	public void setReceiver(JSimProcess newReceiver)
	{
		receiver = newReceiver;
	} // setReceiver

} // class SendingProcess

