/*
 *	A simple simulation of tree processes exchanging messages asynchronously and directly.
 *	The messages know the receiver, not the sender. Neither the sending nor the receiving operation
 *	blocks. If no message is found in the message box by the receiving process, the process just goes on.
 *	The receiving process reads messages from any sender -- the receiver does not specify the sender
 *	so messages from any sender are received.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class AsynchronousCommunicationWithKnownReceiverSimulation
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		SendingProcess sender1;
		SendingProcess sender2;
		ReceivingProcess receiver;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Asynchronous Communication with Known Receiver Simulation");
			receiver = new ReceivingProcess("Receiver", simulation);
			sender1 = new SendingProcess("Sender 1", simulation, receiver, 'A');
			sender2 = new SendingProcess("Sender 2", simulation, receiver, 'B');

			simulation.message("Activating the processes...");
			sender1.activate(0.0);
			sender2.activate(0.0);
			receiver.activate(0.0);

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch

		// We must shutdown the simulation.
		finally
		{
			simulation.shutdown();
		}
	} // main

} // class AsynchronousCommunicationWithKnownReceiverSimulation

