/*
 *	A simple process periodically reading messages from other processes.
 *	If no message is available, null is returned and the receiving process goes on.
 *	After every reading, the process sleeps for a while.
 *
 *	The receiver does not know from which process the new message will be received. The receiver is not even able
 *	to determine the sender after reception because the message does not have this information.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class ReceivingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

//--------------------------------------------------------------------------------------------------

	public ReceivingProcess(String name, JSimSimulation parent) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
	} // constructor

	protected void life()
	{
		JSimMessage message;
		double sleepTime;

		try
		{
			while (true)
			{
				message(myParent.getCurrentTime() + " - " + getName() + ": Trying to receive some data from any process.");
				message = receiveMessageWithoutBlocking();
				if (message == null)
					message(myParent.getCurrentTime() + " - " + getName() + ": No message available now.");
				else
				{
					message(myParent.getCurrentTime() + " - " + getName() + ": Received data: " + message.getData().toString() + ".");
					message(myParent.getCurrentTime() + " - " + getName() + ": Sender is: " + ((message.getSender() == JSimMessage.UNKNOWN_SENDER) ? "<unknown>" : message.getSender().getName()));
				}

				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class ReceivingProcess

