/*
 *	A simple simulation of tree processes exchanging messages asynchronously and indirectly via a message box.
 *	The messages know only the sender, not the receiver (that's the reason why they are sent indirectly).
 *	Neither the sending nor the receiving operation blocks. If no message is found in the message box from a chosen
 *	process by the receiving process, the receiver just goes on.
 *
 *	The receiver decides from which process the new message will be received.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class AsynchronousCommunicationWithKnownSenderSimulation
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		SendingProcess sender1;
		SendingProcess sender2;
		ReceivingProcess receiver;
		JSimMessageBox box;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Asynchronous Communication With Known Sender Simulation");
			box = new JSimMessageBox("Shared Message Box");
			sender1 = new SendingProcess("Sender 1", simulation, box, 'A');
			sender2 = new SendingProcess("Sender 2", simulation, box, 'B');
			receiver = new ReceivingProcess("Receiver", simulation, box, sender1, sender2);

			simulation.message("Activating the processes...");
			sender1.activate(0.0);
			sender2.activate(0.0);
			receiver.activate(0.0);

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch

		// We must shutdown the simulation.
		finally
		{
			simulation.shutdown();
		}
	} // main

} // class AsynchronousCommunicationWithKnownSenderSimulation

