/*
 *	A simple process periodically reading messages sent by two other processes. Since the messages do not have
 *	their receiver specified, they are transfered via a message box. The receiver has to specify both
 *	the message box and the sender (as a filter).
 *	If no message is available from the selected process, null is returned and the receiving process goes on.
 *	After every reading, the process sleeps for a while.
 *
 *	The receiver decides from which process the new message will be received.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class ReceivingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

	private JSimProcess sender1;
	private JSimProcess sender2;
	private JSimMessageBox box;

//--------------------------------------------------------------------------------------------------

	public ReceivingProcess(String name, JSimSimulation parent, JSimMessageBox messageBox, JSimProcess sendingProcess1, JSimProcess sendingProcess2) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		sender1 = sendingProcess1;
		sender2 = sendingProcess2;
		box = messageBox;
	} // constructor

	protected void life()
	{
		JSimMessage message;
		double sleepTime;
		JSimProcess selectedProcess;

		try
		{
			while (true)
			{
				if (JSimSystem.draw(0.5))
					selectedProcess = sender1;
				else
					selectedProcess = sender2;
				message(myParent.getCurrentTime() + " - " + getName() + ": Trying to receive some data from process `" + selectedProcess.getName() + "' indirectly via message box `" + box.getName() + "'.");
				message = receiveMessageWithoutBlocking(box, selectedProcess);
				if (message == null)
					message(myParent.getCurrentTime() + " - " + getName() + ": The selected process has not sent anything.");
				else
				{
					message(myParent.getCurrentTime() + " - " + getName() + ": Received data: " + message.getData().toString() + ".");
					message(myParent.getCurrentTime() + " - " + getName() + ": Sender is `" + ((message.getSender() == JSimMessage.UNKNOWN_SENDER) ? "<unknown>" : message.getSender().getName()) + "'.");
					message(myParent.getCurrentTime() + " - " + getName() + ": Receiver is `" + ((message.getReceiver() == JSimMessage.UNKNOWN_RECEIVER) ? "<unknown>" : message.getReceiver().getName()) + "'.");
				}

				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class ReceivingProcess

