/*
 *	A simple process periodically sending assymetric messages to a message box. After every send,
 *	the process sleeps for a while. The receiver is not known.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class SendingProcess extends JSimProcess
{
	public static final double LAMBDA = 0.5;

	private JSimMessageBox box;
	private char prefix;

//--------------------------------------------------------------------------------------------------

	public SendingProcess(String name, JSimSimulation parent, JSimMessageBox messageBox, char messagePrefix) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		box = messageBox;
		prefix = messagePrefix;
	} // constructor

	protected void life()
	{
		JSimMessageFromSender message;
		int counter = 0;
		double sleepTime;
		String messageData;

		try
		{
			while (true)
			{
				messageData = Character.toString(prefix) + Integer.toString(counter);
				message = new JSimMessageFromSender(this, messageData);
				message(myParent.getCurrentTime() + " - " + getName() + ": Sending a message with data `" + messageData + "' to message box `" + box.getName() + "'.");
				sendMessageWithoutBlocking(message, box);
				message(myParent.getCurrentTime() + " - " + getName() + ": Mesage sent.");
				counter++;
				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class SendingProcess

