/*
 *	A simple simulation of two processes exchanging messages synchronously and directly (process to process).
 *	The messages know both the sender and the receiver. Both the sending and the receiving operations
 *	block. If no message is found in the default message box of the receiving process, the process blocks
 *	until the sender sends it a message. The sender blocks during send if the receiver is not ready to
 *	receive a message. The sender remains blocked until the receiver performs a receive operation.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class SynchronousSymmetricCommunicationSimulation
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		SendingProcess sender;
		ReceivingProcess receiver;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Synchronous Symmetric Communication Simulation");
			receiver = new ReceivingProcess("Receiver", simulation);
			sender = new SendingProcess("Sender", simulation, receiver);

			simulation.message("Activating the processes...");
			sender.activate(0.0);
			receiver.activate(0.0);

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch

		// We must shutdown the simulation.
		finally
		{
			simulation.shutdown();
		}
	} // main

} // class SynchronousSymmetricCommunicationSimulation

