/*
 *	A simple process periodically sending messages to a message box. The send operation is blocking.
 *	If no receiver is ready to receive the message from the box, the sender blocks.
 *	After every insertion, the sending process sleeps for a while.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class SendingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

	private JSimMessageBox box;

//--------------------------------------------------------------------------------------------------

	public SendingProcess(String name, JSimSimulation parent, JSimMessageBox messageBox) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		box = messageBox;
	} // constructor

	protected void life()
	{
		JSimIndirectMessage message;
		int counter = 0;
		double sleepTime;

		try
		{
			while (true)
			{
				message = new JSimIndirectMessage(new Integer(counter));
				message(myParent.getCurrentTime() + " - " + getName() + ": Sending a message with data `" + counter + "' to message box `" + box.getName() + "'.");
				sendMessageWithBlocking(message, box);
				message(myParent.getCurrentTime() + " - " + getName() + ": Mesage sent, sender is running again.");
				counter++;
				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class SendingProcess

