/*
 *	A simple simulation of six processes (3 senders, 3 receivers) exchanging messages synchronously and
 *	indirectly via a message box. The messages know neither the sender nor the receiver.
 *	Both the sending and the receiving operations block.
 *	If no message is found in the message box by the receiving process, the process blocks
 *	until the sender sends it a message. The sender blocks during send if the receiver is not ready to
 *	receive a message. The sender remains blocked until the receiver performs a receive operation.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class SynchronousIndirectCommunicationSimulation
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		JSimMessageBox box;
		SendingProcess sender1;
		SendingProcess sender2;
		SendingProcess sender3;
		ReceivingProcess receiver1;
		ReceivingProcess receiver2;
		ReceivingProcess receiver3;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Synchronous Indirect Communication Simulation");
			box = new JSimMessageBox("Shared Message Box");
			receiver1 = new ReceivingProcess("Receiver 1", simulation, box);
			receiver2 = new ReceivingProcess("Receiver 2", simulation, box);
			receiver3 = new ReceivingProcess("Receiver 3", simulation, box);
			sender1 = new SendingProcess("Sender 1", simulation, box, 'A');
			sender2 = new SendingProcess("Sender 2", simulation, box, 'B');
			sender3 = new SendingProcess("Sender 3", simulation, box, 'C');

			simulation.message("Activating the processes...");
			sender1.activate(0.0);
			sender2.activate(0.0);
			sender3.activate(0.0);
			receiver1.activate(0.0);
			receiver2.activate(0.0);
			receiver3.activate(0.0);

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch

		// We must shutdown the simulation.
		finally
		{
			simulation.shutdown();
		}
	} // main

} // class SynchronousIndirectCommunicationSimulation

