/*
 *	A simple process periodically receiving messages from any process.
 *	If no message is available, the receiver blocks until the selected process sends it a message.
 *	After every reading, the process sleeps for a while.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class ReceivingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

//--------------------------------------------------------------------------------------------------

	public ReceivingProcess(String name, JSimSimulation parent) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
	} // constructor

	protected void life()
	{
		JSimMessage message;
		double sleepTime;

		try
		{
			while (true)
			{
				message(myParent.getCurrentTime() + " - " + getName() + ": Trying to receive some data from any process.");
				message = receiveMessageWithBlocking();
				message(myParent.getCurrentTime() + " - " + getName() + ": Running again.");
				message(myParent.getCurrentTime() + " - " + getName() + ": Received data: " + message.getData().toString());
				message(myParent.getCurrentTime() + " - " + getName() + ": Sender is `" + (message.getSender() == JSimMessage.UNKNOWN_SENDER ? "<unknown>" : message.getSender().getName()) + "'.");
				message(myParent.getCurrentTime() + " - " + getName() + ": Receiver is `" + (message.getReceiver() == JSimMessage.UNKNOWN_RECEIVER ? "<unknown>" : message.getReceiver().getName()) + "'.");

				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class ReceivingProcess

