/*
 *	A simple process periodically sending messages to another process. The send operation is blocking.
 *	If the receiver is not ready to receive the message, the sender blocks.
 *	After every insertion, the sending process sleeps for a while.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class SendingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

	private char prefix;
	private JSimProcess receiver;

//--------------------------------------------------------------------------------------------------

	public SendingProcess(String name, JSimSimulation parent, JSimProcess receivingProcess, char messagePrefix) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		receiver = receivingProcess;
		prefix = messagePrefix;
	} // constructor

	protected void life()
	{
		JSimMessageForReceiver message;
		int counter = 0;
		double sleepTime;
		String messageData;

		try
		{
			while (true)
			{
				messageData = Character.toString(prefix) + Integer.toString(counter);
				message = new JSimMessageForReceiver(receiver, messageData);
				message(myParent.getCurrentTime() + " - " + getName() + ": Sending a message with data `" + messageData + "' to process `" + receiver.getName() + "'.");
				sendMessageWithBlocking(message);
				message(myParent.getCurrentTime() + " - " + getName() + ": Mesage sent, sender is running again.");
				counter++;
				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class SendingProcess

