/*
 *	A simple simulation of three processes (2 senders, 1 receiver) exchanging messages synchronously and
 *	indirectly via a message box. The messages know only the sender, not the receiver.
 *	Both the sending and the receiving operations block.
 *	If no message is found in the message box by the receiving process, the process blocks
 *	until a sender sends it a message. The sender blocks during send if the receiver is not ready to
 *	receive a message from this specific process. The sender remains blocked until the receiver performs
 *	a receive operation.
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.ipc.*;

public class SynchronousCommunicationWithKnownSenderSimulation
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		JSimMessageBox box;
		SendingProcess sender1;
		SendingProcess sender2;
		ReceivingProcess receiver;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Synchronous Communication with Known Sender Simulation");
			box = new JSimMessageBox("Shared Message Box");
			sender1 = new SendingProcess("Sender 1", simulation, box, 'A');
			sender2 = new SendingProcess("Sender 2", simulation, box, 'B');
			receiver = new ReceivingProcess("Receiver", simulation, box, sender1, sender2);

			simulation.message("Activating the processes...");
			sender1.activate(0.0);
			sender2.activate(0.0);
			receiver.activate(0.0);

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch

		// We must shutdown the simulation.
		finally
		{
			simulation.shutdown();
		}
	} // main

} // class SynchronousCommunicationWithKnownSenderSimulation

