/*
 *	Copyright (c) 2000 - 2004, University of West Bohemia,
 *	                           Faculty of Applied Sciences,
 *	                           Department of Computer Science and Engineering,
 *	                           DSS Group.
 *	                           Pilsen, Czech Republic.
 *
 *	Please read the licence agreement carefully before using any part of this software.
 */
package cz.zcu.fav.kiv.jsim;

import java.io.PrintStream;

/**
 *	The JSimInvalidParametersException is thrown whenever a method or a constructor of a J-Sim class
 *	receives arguments that cannot be used. You should never need to create an instance of this class.
 *
 *	@author		<a href="http://home.zcu.cz/~jkacer">Jarda KAER</a>
 *
 *	@version	J-Sim version 0.3.0
 *
 *	@since		J-Sim version 0.0.1
 */
public class JSimInvalidParametersException extends JSimException
{
	/**
	 *	Creates a new JSimInvalidParametersException with a specific information.
	 *
	 *	@param pars	Holds detailed information about specific circumstances of this exception,
	 *			for example method's or a variable's name.
	 */
	public JSimInvalidParametersException(String pars)
	{
		super("You passed one or more parameters with invalid value to a method.", pars);
	} // constructor

	/*
	 *	Method comments are inherited.
	 */
	public void printComment(PrintStream ps)
	{
		ps.println("Your parameters are invalid and have been rejected.");
		ps.println();

		if (getSpecificInfo() != null)
		{
			ps.println("Additional information: " + getSpecificInfo());
			ps.println();
		} // if
	} // printComment

} // class JSimInvalidParametersException

