/*
 *	Copyright (c) 2000 - 2004, University of West Bohemia,
 *	                           Faculty of Applied Sciences,
 *	                           Department of Computer Science and Engineering,
 *	                           DSS Group.
 *	                           Pilsen, Czech Republic.
 *
 *	Please read the licence agreement carefully before using any part of this software.
 */
package cz.zcu.fav.kiv.jsim;

import java.io.PrintStream;

/**
 *	The JSimInvalidProcessStateException is thrown whenever an attempt is made to change a process's state
 *	with JSimProcess.setProcessState() but there is not a transition between the current and the desired
 *	state in the graph of J-Sim process states.
 *	You should never need to create an instance of this class.
 *
 *	@author		<a href="http://home.zcu.cz/~jkacer">Jarda KAER</a>
 *
 *	@version	J-Sim version 0.3.0
 *
 *	@since		J-Sim version 0.1.2
 */
public class JSimInvalidProcessStateException extends JSimException
{
	private int oldState;
	private int newState;

//--------------------------------------------------------------------------------------------------

	/**
	 *	Creates a new JSimInvalidProcessStateException with a specific information.
	 *
	 *	@param pars	Holds detailed information about specific circumstances of this exception,
	 *			for example a method's or a varibale's name.
	 *	@param oldSt	The current state of a process.
	 *	@param newSt	The state that the user or J-Sim attempted to switch to.
	 */
	public JSimInvalidProcessStateException(String param, int oldSt, int newSt)
	{
		super("You attempted to switch the process to a state which is not allowed now.", param);
		oldState = oldSt;
		newState = newSt;
	} // constructor

	/*
	 *	Method comments are inherited.
	 */
	public void printComment(PrintStream ps)
	{
		ps.println("The current state of the process does not allow you to switch it to the state you are requesting.");
		ps.println("Current state: "   + JSimProcess.getStateAsString(oldState) + " (" + oldState + ")");
		ps.println("Requested state: " + JSimProcess.getStateAsString(newState) + " (" + newState + ")");
		ps.println();

		if (getSpecificInfo() != null)
		{
			ps.println("Additional information: " + getSpecificInfo());
			ps.println();
		} // if
	} // printComment

} // class JSimInvalidProcessStateException

