/*
 *	Copyright (c) 2000 - 2004, University of West Bohemia,
 *	                           Faculty of Applied Sciences,
 *	                           Department of Computer Science and Engineering,
 *	                           DSS Group.
 *	                           Pilsen, Czech Republic.
 *
 *	Please read the licence agreement carefully before using any part of this software.
 */
package cz.zcu.fav.kiv.jsim.ipc;

import cz.zcu.fav.kiv.jsim.*;

/**
 *	An indirect message is a message whose sender and receiver are not known. The message cannot be sent directly,
 *	only via a message box. Any process can pick it up from the message box later.
 *
 *	@author		<a href="http://home.zcu.cz/~jkacer">Jarda KAER</a>
 *
 *	@version	J-Sim version 0.3.0
 *
 *	@since		J-Sim version 0.3.0
 */
public class JSimIndirectMessage extends JSimMessage
{
	/**
	 *	Creates a new indirect message with an unknown sender, an unspecified receiver, and the specified
	 *	user data. The default value of message type will be used.
	 *
	 *	@param data	User data that the message will carry.
	 */
	public JSimIndirectMessage(Object data)
	{
		super(JSimMessage.UNKNOWN_SENDER, JSimMessage.UNKNOWN_RECEIVER, data);
	} // constructor

	/**
	 *	Creates a new message with an unknown sender, an unspecified receiver, and the specified user data
	 *	and message type.
	 *
	 *	@param data		User data that the message will carry.
	 *	@param mesageType	The type of the message. Must be non-negative.
	 *
	 *	@exception JSimInvalidParametersException	This exception is thrown out if the message type is
	 *							negative.
	 */
	public JSimIndirectMessage(Object data, int messageType) throws JSimInvalidParametersException
	{
		super(JSimMessage.UNKNOWN_SENDER, JSimMessage.UNKNOWN_RECEIVER, data, messageType);
	} // constructor

} // class JSimIndirectMessage

