/*
 *	Copyright (c) 2000 - 2004, University of West Bohemia,
 *	                           Faculty of Applied Sciences,
 *	                           Department of Computer Science and Engineering,
 *	                           DSS Group.
 *	                           Pilsen, Czech Republic.
 *
 *	Please read the licence agreement carefully before using any part of this software.
 */
package cz.zcu.fav.kiv.jsim.ipc;

import cz.zcu.fav.kiv.jsim.*;

import java.io.PrintStream;

/**
 *	The JSimTooManySemaphoresException is thrown whenever J-Sim cannot add a new semaphore (JSimSemaphore) into
 *	a simulation. You can create at most 9.223.372.036.854.775.807 semaphores within a simulation.
 *	You should never need to create an instance of this class.
 *
 *	@author		<a href="http://home.zcu.cz/~jkacer">Jarda KAER</a>
 *
 *	@version	J-Sim version 0.3.0
 *
 *	@since		J-Sim version 0.3.0
 */
public class JSimTooManySemaphoresException extends JSimException
{
	/**
	 *	Creates a new JSimTooManySemaphoresException with a specific information.
	 *
	 *	@param pars	Holds detailed information about specific circumstances of this exception,
	 *			for example a method's or a variable's name.
	 */
	public JSimTooManySemaphoresException(String pars)
	{
		super("J-Sim cannot add a new semaphore to the simulation.", pars);
	} // constructor

	/*
	 *	Method comments are inherited.
	 */
	public void printComment(PrintStream ps)
	{
		ps.println("J-Sim is not able to add a new semaphore (JSimSemaphore) to the simulation");
		ps.println("because there is no available number for that.");
		ps.println("There can be at most " + Long.MAX_VALUE + " semaphores created in a simulation.");
		ps.println();

		if (getSpecificInfo() != null)
		{
			ps.println("Additional information: " + getSpecificInfo());
			ps.println();
		} // if
	} // printComment

} // class JSimTooManySemaphoresException

